#!/usr/local/bin/python
'''
Console Class
'''
'''
To Do:
    -
'''

from Listener import Listener
from _version import __version__
import time

# =============================================================================
# Console
# 
# Inheriting from `object` (top-level class)
# =============================================================================
class Console(object):
    def __init__(self, args=None):
        '''
        Console Initialization
        Input Attributes:
        -----------------
        self.args -> argparse.Namespace: object holding attributes set
                                         on command-line.
        '''

        # Default Values
        delay = 3.0
        number = 1 # XXX should be = 0
        command = [ ]
        interface = 'eth0'
        listen = False
        host = 'localhost'
        if args:
            delay = float(args.delay) if args.delay else delay
            number = args.number if (args.number != None) else number
            command = args.command if args.command else command
            interface = args.interface if args.interface else interface
            listen = args.listen if args.listen else listen
            host = args.host if args.host else host

        # Input Checks
        if command != [ ]:
            raise NotImplementedError('Issuing commands to hosts has ' +
                    'not been implemented yet')

        # init ...
        if listen:
            self.listener = Listener(interface)
            key = '''
remote          Dly St Dom Pr1  Cl Acc   Var  Pr2       Uniq       SyncT  DlyT  AnnT
===================================================================================='''.strip()


            while number > 0:
                # Report output directly to console
                fmt='%a %b %d %Y %H:%M:%S'
                t = time.time()
                time_str = time.strftime(fmt, time.localtime(t))
                time_msecs = int((t - int(t)) * 1000)

                print time_str + '.%03d ' % (time_msecs) + time.tzname[0]
                print key

                # output data seen in since last iteration
                neighbor_stats = self.listener.ptp_neighbors
                for neighbor in neighbor_stats:
                    print self.listener.ptp_neighbors[neighbor]

                print
                number -= 1
                if number <= 0:
                    exit(0)
                    # No need to wait after the last iteration
                time.sleep(delay)

            # Enter into the interactive environment, exit when q is
            # issued

        else:
            for supplied_command in command:
                command = supplied_command.lower()

                if command == 'rv' or command == 'readvar':
                    None
                    # Assuming to be similar to NTPQ
            # root@raspberrypi:/home/puppet# ntpq -n -c rv -c peers
            #associd=0 status=0615 leap_none, sync_ntp, 1 event, clock_sync,
            #version="ntpd 4.2.6p5@1.2349-o Mon Nov  2 04:29:47 UTC 2015 (1)",
            #processor="armv6l", system="Linux/4.1.17+", leap=00, stratum=3,
            #precision=-20, rootdelay=2.916, rootdisp=60.561,
            #refid=3.44.174.43,
            #reftime=da7f3666.54078831  Mon, Feb 29 2016 21:28:06.328,
            #clock=da7f38cd.57a72dc6  Mon, Feb 29 2016 21:38:21.342,
            #peer=7185, tc=8,
            #mintc=3, offset=9.208, frequency=-48.954, sys_jitter=0.000,
            #clk_jitter=16.919, clk_wander=4.216
                elif command == 'peers':
                    None
                    # Assuming to be similar to NTPQ
            #     remote           refid      st t when poll reach   delay   offset  jitter
            #==============================================================================
            #*useclsifl158.tf 3.199.96.254     2 u   14 1024  377    1.582    0.186   0.919
                else:
                    raise NotImplementedError('Unknown command, \'' +
                            command + '\'')

# __main__.py is executed when the package is instantiated
import argparse

def main():
    parser = argparse.ArgumentParser(prog='ptpop', description='Gain ' +
        'insight into the operations of IEEE 1588 Precision Time Protocol ' +
        'domains on a network. Press the \'q\' key to quit.')

    command_choices=['readvar', 'rv', 'peers']
    parser.add_argument('host', type=str, nargs='?', help='each of the ' +
                        'commands will be sent to the PTP servers ' +
                        'running on the host provided, localhost by ' +
                        'default.')
    parser.add_argument('-c', '--command', type=str, action='append',
                        help='a command to run on the provided host, ' +
                        'i.e. ' + str(command_choices) + ', \'readvar\' ' +
                        'by default. Multiple commands can be issued.')
    parser.add_argument('-i', '--interface', type=str,
                        help='interface to issue commands on or to ' +
                        'observe on in listen mode.')
    parser.add_argument('-l', '--listen', action='store_true',
                        help='don\'t contact any PTP servers, but ' +
                        'report on any services currently observed ' +
                        'on the network, instead.')
    parser.add_argument('-d', '--delay', metavar='SECS.TENTHS', type=str,
                        help='Specifies the delay between screen ' +
                        'updates when interactive. Can be changed while ' +
                        'running using the \'d\' key. Negative ' +
                        'numbers are not allowed. Setting this value ' +
                        'to 0 is the same as issuing the \'-n 1\' ' +
                        'option.')
    parser.add_argument('-n', '--number', metavar='COUNT', type=int,
                        help='Specifies the maximum number of iterations ' +
                        'in interactive mode before ending.')
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s ' + __version__)

    args = parser.parse_args()
    try:
        c = Console(args)

    except Exception as e:
        print type(e).__name__ + ": " + str(e.message)
        exit(-1)

if __name__ == '__main__':
    main()



#!/usr/local/bin/python
'''
Listener Class
'''

import pcap
import thread
from PtpNeighbor import PtpNeighbor
from PtpPacket import PtpPacket

# =============================================================================
# Listener
# 
# Inheriting from `object` (top-level class)
# =============================================================================
class Listener(object):
    def __init__(self, intf=None):
        '''
        Listener Initialization

        Input Attributes:
        ------------------
        intf: network interface the listener should observe
        '''

        # Default Values
        self.ptp_neighbors = dict()

        # Input Checks
        if intf:
            self._pcap = pcap.pcap(name=intf)
            self._pcap.setfilter('dst port 319 or dst port 320')
            # UDP port 319: Sync, Delay_Req, Pdelay_Req, Pdelay_Resp
            # UDP port 320: Follow_Up, Delay_Resp, Pdelay_Resp_Follow_Up, Announce, Management, Signaling
            thread.start_new_thread(self._pcap.loop, (999, self._handle_received_packet))


    def read_from_pcap_file(self, filename):
        pc = pcap.pcap(filename)
        for ts, pkt in pc:
            pn = PtpNeighbor(pkt)
            pn_key = "%s:%d" % (pn.src_addr_str, pn.domain)
            self.ptp_neighbors[pn_key] = pn

    @classmethod
    def from_pcap_file(cls, filename):
        "Initialize Listener with packets read from a pcap"
        l = cls()
        l.read_from_pcap_file(filename)
        return l


# =============================================================================
# Private Module Functions
# =============================================================================
    def _handle_received_packet(self, timestamp, data):
        '''
        Interpret a packet observed on the network interface

        Keyword Arguments:
        ------------------
        #'''
        pp = PtpPacket(data)
        ptp_neighbor_key = "%s:%d" % (pp.src_addr_str, pp.domain)

        try:
            ptp_neighbor = self.ptp_neighbors[ptp_neighbor_key]
            if pp.ptp_control == 5:
                ptp_neighbor.new_announce_message(data)
            elif pp.ptp_control == 0:
                ptp_neighbor.new_sync_message(data)
        except KeyError:
            if pp.ptp_control == 5:
                self.ptp_neighbors[ptp_neighbor_key] = PtpNeighbor(data)


#==============================================================================
# Class Test
#==============================================================================
import unittest
import time

class TestListener(unittest.TestCase):
    def test_new_with_mock_data(self):
        l = Listener.from_pcap_file('single_ptp_announce_packet_from_2_to_129.pcap')
        l.read_from_pcap_file('single_ptp_announce_packet_from_3_to_129.pcap')
        l.read_from_pcap_file('single_ptp_announce_packet_from_4_to_129.pcap')

        ptp_neighbor_keys = iter(l.ptp_neighbors)
        for ptp_neighbor_key in ptp_neighbor_keys:
            actual = str(l.ptp_neighbors[ptp_neighbor_key])
            expected = str()
            if ptp_neighbor_key == '192.168.1.2:129':
                expected = '192.168.1.2     E2E  - 129 128   6 0x21 15652 128 001c73ffffb53519   -     -     -  '
            elif ptp_neighbor_key == '192.168.1.3:129':
                expected = '192.168.1.3     E2E  - 129 128   6 0x21 15652 128 001c73ffff1935b5   -     -     -  '
            elif ptp_neighbor_key == '192.168.1.4:129':
                expected = '192.168.1.4     E2E  - 129 128   6 0x21 15652 128 001c73ffff35b519   -     -     -  '
            else:
                self.fail(msg='Unknown key in ptp_neighbor_keys: ' +
                        ptp_neighbor_key)
            self.assertEqual(actual, expected)

    def test_listening_to_intf(self):
        l = Listener('lo')

        pcap_file = pcap.pcap('single_ptp_announce_packet_from_4_to_129.pcap')
        pc = pcap.pcap(name='lo')
        for ts, pkt in pcap_file:
            pc.sendpacket(str(pkt))

        # Cheap fix to the race condition
        time.sleep(0.1)
        ptp_neighbor_key = iter(l.ptp_neighbors)
        actual = str(l.ptp_neighbors[ptp_neighbor_key.next()])
        expected = "192.168.1.4     E2E  - 129 128   6 0x21 15652 128 001c73ffff35b519   -     -     -  "
        self.assertEqual(actual, expected)

if __name__ == '__main__':
    # Test class
    unittest.main()



#!/usr/bin/env python
# -*- coding: ascii -*-

# The __init__.py is loaded (only) when importing the package

"""
Package Ptpop
Docstrings: http://www.python.org/dev/peps/pep-0257/
"""

__author__ = 'Andrew Burnheimer (Andrew.Burnheimer@nbcuni.com)'
__copyright__ = 'Copyright (c) 2016 Andrew Burnheimer'
__license__ = 'Creative Commons Attribution and ShareAlike'

from Console import Console
from Listener import Listener
from PtpNeighbor import PtpNeighbor
from PtpPacket import PtpPacket
from SyncMessage import SyncMessage
from FollowUpMessage import FollowUpMessage
from AnnounceMessage import AnnounceMessage
from _version import __version__



#!/usr/local/bin/python
'''
PtpNeighbor Class
'''

import pcap
import re
import datetime
from AnnounceMessage import AnnounceMessage
from SyncMessage import SyncMessage

# =============================================================================
# PtpNeighbor
# 
# Inheriting from `object` (top-level class)
# =============================================================================
class PtpNeighbor(object):
    def __init__(self, pkt):
        '''
        PtpPacket Initialization

        Input Attributes:
        ------------------
        pkt: PTP Announce packet to derive PTP details of this neighbor
             from
        '''

        self._sync_period = None
        self._delay_period = None
        self._announce_period = None
        self._time_of_last_sync = 0
        self._time_of_last_delay = 0
        self._time_of_last_announce = 0
        self.new_announce_message(pkt)

    @property
    def sync_period(self):
        """Get the latest calculated sync_period"""
        return self._sync_period

    @property
    def delay_period(self):
        """Get the latest calculated delay_period"""
        return self._delay_period

    @property
    def announce_period(self):
        """Get the latest calculated announce_period"""
        return self._announce_period

    def new_announce_message(self, pkt):
        '''
        Take note of an announce message from this neighbor, to derive
        its periodicity
        '''
        msg = AnnounceMessage(pkt)
        self._update_announce_params(msg)
        now = datetime.datetime.now()
        if self._time_of_last_announce != 0:
            self._announce_period = (now - self._time_of_last_announce).total_seconds()
        self._time_of_last_announce = now

    def new_sync_message(self, pkt):
        '''
        Take note of a sync message from this neighbor, to derive the
        its periodicity
        '''
        msg = SyncMessage(pkt)
        now = datetime.datetime.now()
        if self._time_of_last_sync != 0:
            self._sync_period = (now - self._time_of_last_sync).total_seconds()
        self._time_of_last_sync = now


    def _update_announce_params(self, msg):
        '''
        Update all parameters for this neighbor using info. contained
        in the announce message
        '''
        # According to the pcap data, set this objects' properties
        self.src_addr_str = msg.ipv4_src_str
        # Obliged to update delay_mode after P_Delay_Req received 
        self.delay_mode = "E2E"
        # Obliged to update step_mode after sync received 
        self.step_mode = None
        self.domain = msg.ipv4_dst % 256
        self.priority1 = msg.ptp_bmca_priority1
        self.clock_class = msg.ptp_bmca_gm_clock_class
        self.accuracy = msg.ptp_bmca_gm_clock_acc
        self.variance = msg.ptp_bmca_gm_clock_var
        self.priority2 = msg.ptp_bmca_priority2
        self.uniq_id = msg.ptp_clock_id

    def __str__(self):
        stats_str = str()
        stats_str += ( '%-15s' % self.src_addr_str ) if self.src_addr_str != None else ''
        stats_str += ( ' %3s' % self.delay_mode ) if self.delay_mode != None else ''
        stats_str += ( ' %2d' % self.step_mode ) if self.step_mode != None else '  -'
        stats_str += ( ' %3d' % self.domain ) if self.domain != None else ''
        stats_str += ( ' %3d' % self.priority1 ) if self.priority1 != None else ''
        stats_str += ( ' %3d' % self.clock_class ) if self.clock_class != None else ''
        stats_str += ( ' 0x%2x' % self.accuracy ) if self.accuracy != None else ''
        stats_str += ( ' %5d' % self.variance ) if self.variance != None else ''
        stats_str += ( ' %3d' % self.priority2 ) if self.priority2 != None else ''

        trimmed_uniq_id = str()
        exp = re.compile('0x([a-fA-F0-9]+)')
        if self.uniq_id != None:
            match = exp.findall(self.uniq_id)
            trimmed_uniq_id = match[0]
        stats_str += ( ' % 16s' % trimmed_uniq_id )

        stats_str += ( ' % 2.2f' % self.sync_period ) if self.sync_period != None else '   -  '
        stats_str += ( ' % 2.2f' % self.delay_period ) if self.delay_period != None else '   -  '
        stats_str += ( ' % 2.2f' % self.announce_period ) if self.announce_period != None else '   -  '
        return stats_str


#==============================================================================
# Class Test
#==============================================================================
import unittest
import time

class TestPtpNeighbor(unittest.TestCase):
    def test_new_from_pcap_file(self):
        pc = pcap.pcap('single_ptp_announce_packet.pcap')
        ts, pkt = pc.next()
        pn = PtpNeighbor(pkt)
        self.assertEqual(pn.src_addr_str, '192.168.1.2')
        self.assertEqual(pn.delay_mode, 'E2E')
        self.assertIsNone(pn.step_mode)
        self.assertEqual(pn.domain, 129)
        self.assertEqual(pn.priority1, 128)
        self.assertEqual(pn.clock_class, 6)
        self.assertEqual(pn.accuracy, 33)
        self.assertEqual(pn.variance, 15652)
        self.assertEqual(pn.priority2, 128)
        self.assertEqual(pn.uniq_id, '0x001c73ffffb53519')
        self.assertIsNone(pn.sync_period)
        self.assertIsNone(pn.delay_period)
        self.assertIsNone(pn.announce_period)

    def test_pcap_file_object_as_string(self):
        pc = pcap.pcap('single_ptp_announce_packet.pcap')
        ts, pkt = pc.next()
        pn = PtpNeighbor(pkt)
        actual = str(pn)
        expected = "192.168.1.2     E2E  - 129 128   6 0x21 15652 128 001c73ffffb53519   -     -     -  "
        self.assertEqual(actual, expected)

    def test_updating_announce_period(self):
        pc = pcap.pcap('single_ptp_announce_packet.pcap')
        ts, pkt = pc.next()
        pn = PtpNeighbor(pkt)
        self.assertIsNone(pn.announce_period)
        time.sleep(0.667)
        pc = pcap.pcap('single_ptp_announce_packet.pcap')
        ts, pkt = pc.next()
        pn.new_announce_message(pkt)
        self.assertIsNotNone(pn.announce_period)
        actual = str(pn)
        expected = "192.168.1.2     E2E  - 129 128   6 0x21 15652 128 001c73ffffb53519   -     -    0.67"
        self.assertEqual(actual, expected)

    def test_updating_sync_period(self):
        ann_pc = pcap.pcap('single_ptp_announce_packet.pcap')
        ts, pkt = ann_pc.next()
        pn = PtpNeighbor(pkt)
        self.assertIsNone(pn.sync_period)

        sync_pc = pcap.pcap('single_ptp_sync_packet.pcap')
        ts, pkt = sync_pc.next()
        pn.new_sync_message(pkt)
        time.sleep(0.333)
        sync_pc = pcap.pcap('single_ptp_sync_packet.pcap')
        ts, pkt = sync_pc.next()
        pn.new_sync_message(pkt)
        self.assertIsNotNone(pn.sync_period)
        actual = str(pn)
        expected = "192.168.1.2     E2E  - 129 128   6 0x21 15652 128 001c73ffffb53519  0.34   -     -  "
        self.assertEqual(actual, expected)


if __name__ == '__main__':
    # Test class
    unittest.main()



#!/usr/local/bin/python
'''
SyncMessage Class
'''

import struct

# =============================================================================
# SyncMessage
# 
# Inheriting from `object` (top-level class)
# =============================================================================
class SyncMessage(object):
    ENET_DST_LEN = 6
    ENET_SRC_LEN = 6
    ENET_TYPE_LEN = 2

    IPV4_VER_LEN = 1
    IPV4_DIFF_SERV_LEN = 1
    IPV4_TOTAL_LEN = 2
    IPV4_ID_LEN = 2
    IPV4_FLAGS_OFFSET_LEN = 2
    IPV4_TTL_LEN = 1

    IPV4_PROTOCOL_LEN = 1
    IPV4_HDR_CHKSUM_LEN = 2
    IPV4_SRC_LEN = 4
    IPV4_DST_LEN = 4

    UDP_SRC_LEN = 2
    UDP_DST_LEN = 2
    UDP_LEN_LEN = 2
    UDP_CHKSUM_LEN = 2

    PTP_MESSAGE_FLAGS = 2
    PTP_MESSAGE_LEN = 2
    PTP_SUBDOMAIN = 1
    # 1 byte padding
    PTP_FLAGS = 2
    PTP_CORRECTION = 8

    # 4 bytes padding 
    PTP_CLOCK_ID = 8
    PTP_SOURCE_PORT_ID = 2
    PTP_SEQ_ID = 2
    PTP_CONTROL = 1
    PTP_LOG_MESSAGE_PER = 1
    PTP_ORIGIN_TIMESTAMP_S = 6
    PTP_ORIGIN_TIMESTAMP_NS = 4

    def __init__(self, bffr):
        '''
        SyncMessage Initialization

        Input Attributes:
        ------------------
        bffr: captured packet string to decode
        '''

        cursor = 0

        self.enet_dst, = struct.unpack(">Q", '\x00\x00' + bffr[cursor:cursor + self.ENET_DST_LEN])
        self.enet_dst_str = self._Binary_MAC_to_String(bffr[cursor:cursor + self.ENET_DST_LEN])
        cursor += self.ENET_DST_LEN

        self.enet_src, = struct.unpack(">Q", '\x00\x00' + bffr[cursor:cursor + self.ENET_SRC_LEN])
        self.enet_src_str = self._Binary_MAC_to_String(bffr[cursor:cursor + self.ENET_SRC_LEN])
        cursor += self.ENET_SRC_LEN

        self.enet_type = self._Binary_Repr(bffr[cursor:cursor + self.ENET_TYPE_LEN])
        cursor += self.ENET_TYPE_LEN

        self.ipv4_ver = 0
        self.ipv4_hdr_len = 0
        ipv4_ver = bffr[cursor:cursor + self.IPV4_VER_LEN]
        if ipv4_ver.encode("hex") == b'45':
            self.ipv4_ver = 4
            self.ipv4_hdr_len = 20
        cursor += self.IPV4_VER_LEN


        self.ipv4_diff_serv = self._Binary_Repr(bffr[cursor:cursor + self.IPV4_DIFF_SERV_LEN])
        cursor += self.IPV4_DIFF_SERV_LEN

        self.ipv4_total_len, = struct.unpack(">H", bffr[cursor:cursor + self.IPV4_TOTAL_LEN])
        cursor += self.IPV4_TOTAL_LEN

        self.ipv4_id, = struct.unpack(">H", bffr[cursor:cursor + self.IPV4_ID_LEN])
        cursor += self.IPV4_ID_LEN

        self.ipv4_flags_offset = self._Binary_Repr(bffr[cursor:cursor + self.IPV4_FLAGS_OFFSET_LEN])
        cursor += self.IPV4_FLAGS_OFFSET_LEN

        self.ipv4_ttl, = struct.unpack(">B", bffr[cursor:cursor + self.IPV4_TTL_LEN])
        cursor += self.IPV4_TTL_LEN

        self.ipv4_protocol, = struct.unpack(">B", bffr[cursor:cursor + self.IPV4_PROTOCOL_LEN])
        cursor += self.IPV4_PROTOCOL_LEN

        self.ipv4_hdr_chksum = self._Binary_Repr(bffr[cursor:cursor + self.IPV4_HDR_CHKSUM_LEN])
        cursor += self.IPV4_HDR_CHKSUM_LEN

        self.ipv4_src, = struct.unpack(">L", bffr[cursor:cursor + self.IPV4_SRC_LEN])
        self.ipv4_src_str = self._Binary_IPV4_to_String(bffr[cursor:cursor + self.IPV4_SRC_LEN])
        cursor += self.IPV4_SRC_LEN

        self.ipv4_dst, = struct.unpack(">L", bffr[cursor:cursor + self.IPV4_DST_LEN])
        self.ipv4_dst_str = self._Binary_IPV4_to_String(bffr[cursor:cursor + self.IPV4_DST_LEN])
        cursor += self.IPV4_DST_LEN


        self.udp_src, = struct.unpack(">H", bffr[cursor:cursor + self.UDP_SRC_LEN])
        cursor += self.UDP_SRC_LEN

        self.udp_dst, = struct.unpack(">H", bffr[cursor:cursor + self.UDP_DST_LEN])
        cursor += self.UDP_DST_LEN

        self.udp_len, = struct.unpack(">H", bffr[cursor:cursor + self.UDP_LEN_LEN])
        cursor += self.UDP_LEN_LEN

        self.udp_chksum = self._Binary_Repr(bffr[cursor:cursor + self.UDP_CHKSUM_LEN])
        cursor += self.UDP_CHKSUM_LEN


        self.ptp_message_flags = self._Binary_Repr(bffr[cursor:cursor + self.PTP_MESSAGE_FLAGS])
        cursor += self.PTP_MESSAGE_FLAGS

        self.ptp_message_len, = struct.unpack(">H", bffr[cursor:cursor + self.PTP_MESSAGE_LEN])
        cursor += self.PTP_MESSAGE_LEN

        self.ptp_subdomain, = struct.unpack(">B", bffr[cursor:cursor + self.PTP_SUBDOMAIN])
        cursor += self.PTP_SUBDOMAIN

        cursor += 1 # padding

        self.ptp_flags = self._Binary_Repr(bffr[cursor:cursor + self.PTP_FLAGS])
        cursor += self.PTP_FLAGS

        self.ptp_correction = self._Binary_Repr(bffr[cursor:cursor + self.PTP_CORRECTION])
        cursor += self.PTP_CORRECTION


        cursor += 4 # padding

        self.ptp_clock_id = self._Binary_Repr(bffr[cursor:cursor + self.PTP_CLOCK_ID])
        cursor += self.PTP_CLOCK_ID

        self.ptp_source_port_id, = struct.unpack(">H", bffr[cursor:cursor + self.PTP_SOURCE_PORT_ID])
        cursor += self.PTP_SOURCE_PORT_ID

        self.ptp_seq_id, = struct.unpack(">H", bffr[cursor:cursor + self.PTP_SEQ_ID])
        cursor += self.PTP_SEQ_ID

        self.ptp_control, = struct.unpack(">B", bffr[cursor:cursor + self.PTP_CONTROL])
        cursor += self.PTP_CONTROL

        self.ptp_log_message_per, = struct.unpack(">B", bffr[cursor:cursor + self.PTP_LOG_MESSAGE_PER])
        cursor += self.PTP_LOG_MESSAGE_PER

        self.ptp_origin_timestamp_s = self._Binary_Repr(bffr[cursor:cursor + self.PTP_ORIGIN_TIMESTAMP_S])
        cursor += self.PTP_ORIGIN_TIMESTAMP_S

        self.ptp_origin_timestamp_ns = self._Binary_Repr(bffr[cursor:cursor + self.PTP_ORIGIN_TIMESTAMP_NS])
        cursor += self.PTP_ORIGIN_TIMESTAMP_NS


    def _Binary_MAC_to_String(self, bin_data):
        return "%02x:%02x:%02x:%02x:%02x:%02x" % struct.unpack("BBBBBB",bin_data)

    def _Binary_Repr(self, bin_data):
        ret_str = str()
        ret_str += "0x"
        for byte in bin_data:
            ret_str += "%02x" % struct.unpack("B", byte)
        return ret_str

    def _Binary_IPV4_to_String(self, bin_data):
        return "%d.%d.%d.%d" % struct.unpack("BBBB",bin_data)


    def __str__(self):
        stats_str = str()
        stats_str += '<ptp sync ID: '
        stats_str += self.ptp_clock_id + ' '
        stats_str += self.ipv4_src_str + ' -> '
        stats_str += self.ipv4_dst_str + ' seq: '
        stats_str += str(self.ptp_seq_id) + '>'
        return stats_str


#==============================================================================
# Class Test
#==============================================================================
import pcap
import unittest

class TestSyncMessage(unittest.TestCase):
    def test_new_from_pcap_file(self):
        pc = pcap.pcap('single_ptp_sync_packet.pcap')
        ts, pkt = pc.next()
        am = SyncMessage(pkt)
        self.assertEqual(am.enet_dst, int('01005e000181', 16))
        self.assertEqual(am.enet_dst_str, '01:00:5e:00:01:81')
        self.assertEqual(am.enet_src, int('001c73b5351d', 16))
        self.assertEqual(am.enet_src_str, '00:1c:73:b5:35:1d')
        self.assertEqual(am.enet_type, '0x0800')

        self.assertEqual(am.ipv4_ver, 4)
        self.assertEqual(am.ipv4_hdr_len, 20)
        self.assertEqual(am.ipv4_diff_serv, '0x54')
        self.assertEqual(am.ipv4_total_len, 72)
        self.assertEqual(am.ipv4_id, 0)
        self.assertEqual(am.ipv4_flags_offset, '0x0000')
        self.assertEqual(am.ipv4_ttl, 1)
        self.assertEqual(am.ipv4_protocol, 17)
        self.assertEqual(am.ipv4_hdr_chksum, '0x1626')
        self.assertEqual(am.ipv4_src, int('c0a80102', 16))
        self.assertEqual(am.ipv4_src_str, '192.168.1.2')
        self.assertEqual(am.ipv4_dst, int('e0000181', 16))
        self.assertEqual(am.ipv4_dst_str, '224.0.1.129')

        self.assertEqual(am.udp_src, 319)
        self.assertEqual(am.udp_dst, 319)
        self.assertEqual(am.udp_len, 52)
        self.assertEqual(am.udp_chksum, '0x0d3f')

        self.assertEqual(am.ptp_message_flags, '0x0002')
        self.assertEqual(am.ptp_message_len, 44)
        self.assertEqual(am.ptp_subdomain, 0)
        self.assertEqual(am.ptp_flags, '0x0200')
        self.assertEqual(am.ptp_correction, '0x0000000000000000')
        self.assertEqual(am.ptp_clock_id, '0x001c73ffffb53519')

        self.assertEqual(am.ptp_source_port_id, 4)
        self.assertEqual(am.ptp_seq_id, 41089)
        self.assertEqual(am.ptp_control, 0)
        self.assertEqual(am.ptp_log_message_per, 255)
        self.assertEqual(am.ptp_origin_timestamp_s, '0x000000000000')
        self.assertEqual(am.ptp_origin_timestamp_ns, '0x00000000')

    def test_str_from_pcap_file(self):
        pc = pcap.pcap('single_ptp_sync_packet.pcap')
        ts, pkt = pc.next()
        actual = str(SyncMessage(pkt))
        expected = '<ptp sync ID: 0x001c73ffffb53519 192.168.1.2 -> 224.0.1.129 seq: 41089>'
        self.assertEqual(actual, expected)


if __name__ == '__main__':
    # Test class
    unittest.main()



from setuptools import setup, find_packages

setup(
    name="ptpop",
    version=open("ptpop/_version.py").readlines()[-1].split()[-1].strip("\"'"),
    packages=find_packages(),
    author='Andrew Burnheimer',
    author_email='Andrew.Burnheimer@nbcuni.com',
    url='http://github.com/andrewburnheimer/ptpop',
    description='top for PTP',
    license='ASLv2',
    install_requires=['pypcap'],
    entry_points={
        'console_scripts': [
            'ptpop = ptpop.Console:main'
        ]
    }
)
